﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ConditionActionEncoding {
  [Item("XCSAfterCopyingParentOperator", "Description missing")]
  [StorableClass]
  public class XCSAfterCopyingParentOperator : SingleSuccessorOperator {

    #region Parameter Properties
    public IValueLookupParameter<IntValue> NumerosityParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["Numerosity"]; }
    }
    public IValueLookupParameter<IntValue> ExperienceParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["Experience"]; }
    }
    public IValueLookupParameter<BoolValue> InsertInPopulationParameter {
      get { return (IValueLookupParameter<BoolValue>)Parameters["InsertInPopulation"]; }
    }
    public IValueLookupParameter<IntValue> TempIDParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["TempID"]; }
    }
    #endregion

    [StorableConstructor]
    protected XCSAfterCopyingParentOperator(bool deserializing) : base(deserializing) { }
    protected XCSAfterCopyingParentOperator(XCSAfterCopyingParentOperator original, Cloner cloner)
      : base(original, cloner) {
    }
    public XCSAfterCopyingParentOperator()
      : base() {
      Parameters.Add(new ValueLookupParameter<IntValue>("Numerosity"));
      Parameters.Add(new ValueLookupParameter<IntValue>("Experience"));
      Parameters.Add(new ValueLookupParameter<BoolValue>("InsertInPopulation"));
      Parameters.Add(new ValueLookupParameter<IntValue>("TempID"));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new XCSAfterCopyingParentOperator(this, cloner);
    }

    public override IOperation Apply() {
      NumerosityParameter.ActualValue = new IntValue(1);
      ExperienceParameter.ActualValue = new IntValue(0);
      InsertInPopulationParameter.ActualValue = new BoolValue(true);
      TempIDParameter.ActualValue = new IntValue(-1);
      return base.Apply();
    }
  }
}
